//+--------------------------------------------------------------------------------------+
//|              Extrapolator_Method_4 Parabolic_LWMA.mq4 |
//|                                      Order of linear predication HN Burg          |
//|                                      BurgWin 1 Hamming  2 Parabolic              
// // MA_Method= 0: SMA - Simple Moving Average
// MA_Method= 1: EMA - Exponential Moving Average
// MA_Method= 2: Wilder - Wilder Exponential Moving Average
// MA_Method= 3: LWMA - Linear Weighted Moving Average  
// Price = 6 - Weighted Close = (High+Low+Close*2)/4
// Linear Weighted Moving Average (LWMA)
//
//In the case of weighted moving average, 
// the latest data is of more value than more early data. 
//Weighted moving average is calculated by multiplying each one of 
//the closing prices within the considered series, by a certain weight coefficient.
//
//LWMA = SUM(Close(i)*i, N)/SUM(i, N)
// Where:  SUM(i, N)  is the total sum of weight coefficients.
//+--------------------------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Aqua
#property indicator_color2 Salmon
#property indicator_width1 3
#property indicator_width2 3

//Global constants
#define pi 3.141592653589793238462643383279502884197169399375105820974944592

//Input parameters
extern int     Method   =4;     //Extrapolation method

// Method 1  it trigomometric series
// Use linear prediction
// Method 2: ACF(x,no,a);break;
// Method 3: WBurg(x,no,BurgWin,a);break;
// Method 4: HNBurg(x,no,a);break;
// Method 5: Geom(x,no,a);break;
// Method 6: MCov

extern int     LastBar  =0;    //Last bar in the past data
extern int     PastBars =1700; // Last Bars
// Modified from 100 to 1200

extern double  LPOrder  =0.67;  //Order of linear prediction model; 0 to 1
// A affiner de 0.6  0.7

//LPOrder*PastBars specifies the number of prediction coefficients a[1..LPOrder*PastBars] where a[0]=1
extern int     FutBars  =289;   //Number of bars to predict; for LP is set at PastBars-Order-1 

// extern int     HarmNo   =20;    //Number of frequencies for Method 1; HarmNo=0 computes PastBars harmonics
extern int     HarmNo   =0;    //Number of frequencies for Method 1; HarmNo=0 computes PastBars harmonics

extern double  FreqTOL  =0.0001;//Tolerance of frequency calculation for Method 1
//FreqTOL > 0.001 may not converge

// Weighted Burg Method   
extern int     BurgWin  =2;     //Windowing function for Weighted Burg Method; 0=no window 1=Hamming 2=Parabolic

// Utilisation iMA 
// First TRY 26
extern double MATrendPeriod=3;
static string   indicatorName;
extern color  TextColor = Pink;

//Indicator buffers
double pv[];
double fv[];
double calculA[];

//Global variables
double ETA,INFTY,SMNO;
//Global variables
int np,nf,lb,no,it,i1,t1,t2,t8,last,signal2;
string signal;

int init()
{
   lb=LastBar;
   np=PastBars;
   no=LPOrder*PastBars;
   nf=FutBars;
   if(Method>1) nf=np-no-1;
   if(HarmNo==0) HarmNo=np;
   
   IndicatorBuffers(4);
   SetIndexBuffer(0,pv);
   SetIndexBuffer(1,fv);
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,2);
   SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,2);
   SetIndexShift(0,-lb);            //past data vector 0..np-1; 0 corresponds to bar=lb
   SetIndexShift(1,nf-lb);          //future data vector i=0..nf; nf corresponds to bar=lb
   
   indicatorName = "Extrapolator_HNBurg_Method_Parabolic_SMA_Typical2T1";
      
   ObjectCreate( indicatorName, OBJ_LABEL, 0,0,0,0,0,0,0);
   ObjectSet( indicatorName, OBJPROP_CORNER, 0);
   ObjectSet( indicatorName, OBJPROP_XDISTANCE, 5);
   ObjectSet( indicatorName, OBJPROP_YDISTANCE, 15);
   ObjectSetText( indicatorName, "", 12, "Arial", TextColor);
   

   IndicatorShortName(indicatorName);
   
   return(0);
}

int deinit()
{
	ObjectDelete(indicatorName);
	return(0);
}

int start()
{
   ArrayInitialize(pv,EMPTY_VALUE);
   ArrayInitialize(fv,EMPTY_VALUE);  
   ArrayInitialize(calculA,EMPTY_VALUE);  
   ArrayResize(calculA,np);
   
//Find average of past values
   double av=0.0;
   for(int i=0;i<np-1;i++) { 
       
      //av+=Open[i+lb];
      // calcul Average + Ask
      calculA[i+lb]=iMA(NULL,0,MATrendPeriod,0,MODE_LWMA,PRICE_WEIGHTED,i+lb);
      av +=calculA[i+lb];
      }
      // Ajout du Signal Current Ask
      av += Ask ;
      
   av/=np;
   
//Prepare data
   if(Method==1)
   {
      for(i=0;i<np;i++)
      {
         pv[i]=av;
         if(i<=nf) fv[i]=av;
      }
   }
   else
   {
      double a[],x[];
      ArrayResize(a,no+1);
      ArrayResize(x,np);
      for(i=0;i<np-1;i++)
          {
           x[np-1-i]= calculA[i+lb]-av;
           //x[np-1-i]=Open[i+lb]-av;
          }
          // Ajout du Signal Current Ask
          //x[0]= Ask -av;
   }

//Select extrapolation method
   switch(Method)
   {
      case 1: //Fit trigomometric series
         break;
      //Use linear prediction
      case 2: break;
      case 3: break;
      case 4: HNBurg(x,no,a);break;
      case 5: break;
      case 6:
         bool stop=0;
         for(i=no;i>=1;i--) a[i]=a[i-1];
   }
   
//Calculate linear predictions
   if(Method>1)
   {
      for(int n=no;n<np+nf;n++)
      {
         double sum=0.0;
         for(i=1;i<=no;i++)
            if(n-i<np) sum-=a[i]*x[n-i];
            else sum-=a[i]*fv[n-i-np+1];
         if(n<np) pv[np-1-n]=sum;
         else fv[n-np+1]=sum;
      }
      fv[0]=pv[0];
   
      for(i=0;i<np-no;i++)
      {
         pv[i]+=av;
         fv[i]+=av;
      }
   }
   
//Reorder the predicted vector
   for(i=0;i<=(nf-1)/2;i++)
   {
      double tmp=fv[i];
      fv[i]=fv[nf-i];
      fv[nf-i]=tmp;
   } 
   
     last=FutBars;
  signal = "=>NEUTRAL "; 
  signal2=0;
           
           if ( fv[last] >= (Ask +0.0019) )    { // 19 pips
                signal = "=> Hyper-LONG: ";
                signal2=4;
               }  
           if ( fv[last] <= (Bid -0.0019) )  { // 10 pips
               signal = "=> Hyper-SHORT ";
                  signal2=-4;
               }      
            if ((fv[last] >= (Ask +0.0012)) && (fv[last] < (Ask +0.0019)) )  { 
                    signal = "=> SUPER-LONG: ";  
                    signal2=3;
                }                                 
           if ((fv[last] >= (Ask +0.0007)) && (fv[last] < (Ask +0.0012)) ) { 
                    signal = "=> VERYLONG: "; 
                    signal2=2;
                 }                  
           if ((fv[last] > (Ask +0.0003)) &&  (fv[last] < (Ask +0.0007)))  {       
                   signal = "=> LONG: ";
                   signal2=1;
                }   
           if ((fv[last] <= (Bid -0.0012)) && (fv[last] < (Bid -0.0019)))  { 
               signal = "=> SUPER-SHORT: ";  
               signal2=-3;           
               }    
          if ((fv[last] <= (Bid -0.0007)) && (fv[last] < (Bid -0.0012)))  { 
               signal = "=> VERYSHORT: ";
               signal2=-2;
               }
          if ((fv[last] < (Bid -0.0003)) && (fv[last] < (Bid -0.0007)))  { 
               signal = "=> SHORT "; 
               signal2=-1;
               }
      // Display the last future value   
      Comment("\n\Exp10:"+fv[10]," Exp.20:"+fv[20]+" Exp.30:"+fv[30]+signal);
  
      
    return(signal2); 
   // return(0); 
   
  //  return(0); 
}

//+-------------------------------------------------------------------------+
void HNBurg(double x[], int p, double& a[])
{
   int n=ArraySize(x);
   double df[],db[];
   ArrayResize(df,n);
   ArrayResize(db,n);
   int i,k,kh,ki;
   double w,tmp,num,den,r;
   for(i=0;i<n;i++)
   {
      df[i]=x[i];
      db[i]=x[i];
   }
   //Main loop
   for(k=1;k<=p;k++)
   {
      //Calculate reflection coefficient
      num=0.0;
      den=0.0;
      if(k==1)
      {
         for(i=2;i<n;i++)
         {
            w=x[i-1]*x[i-1];
            num+=w*x[i-1]*(x[i]+x[i-2]);
            den+=w*x[i-1]*x[i-1];
         }
         r=-num/den/2.0;
         if(r>1) r=1.0;
         if(r<-1.0) r=-1.0;
      }
      else
      {
         w=0.0;
         for(i=1;i<k;i++) w+=x[i]*x[i];
         for(i=k;i<n;i++)
         {
            num+=w*df[i]*db[i-1];
            den+=w*(df[i]*df[i]+db[i-1]*db[i-1]);
            w=w+x[i]*x[i]-x[i-k+1]*x[i-k+1];
         }
         r=-2.0*num/den;
      }
      //Calculate prediction coefficients
      a[k]=r;
      kh=k/2;
      for(i=1;i<=kh;i++)
	   {
	     ki=k-i;  
	     tmp=a[i];
	     a[i]+=r*a[ki];
	     if(i!=ki) a[ki]+=r*tmp;
	   }
	   if(k<p)
         //Calculate new residues
         for(i=n-1;i>=k;i--)
         {
            tmp=df[i];
            df[i]+=r*db[i-1];
            db[i]=db[i-1]+r*tmp;
         }
   }
}

